/*---------------------------------------------------------------------------*\
 *           interactive networked Virtual Reality system (inVRs)            *
 *                                                                           *
 *    Copyright (C) 2005-2009 by the Johannes Kepler University, Linz        *
 *                                                                           *
 *                            www.inVRs.org                                  *
 *                                                                           *
 *              contact: canthes@inVRs.org, rlander@inVRs.org                *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                License                                    *
 *                                                                           *
 * This library is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU Library General Public License as published    *
 * by the Free Software Foundation, version 2.                               *
 *                                                                           *
 * This library is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Library General Public License for more details.                          *
 *                                                                           *
 * You should have received a copy of the GNU Library General Public         *
 * License along with this library; if not, write to the Free Software       *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                Changes                                    *
 *                                                                           *
 *                                                                           *
 *                                                                           *
 *                                                                           *
\*---------------------------------------------------------------------------*/

#ifndef _MYBINARYMESSAGE_H
#define _MYBINARYMESSAGE_H

// fix problems with Visual Studio 2008
#ifdef WIN32
#include <stdio.h>
#endif
#include <OpenSG/OSGBaseTypes.h>
#include <OpenSG/OSGBinaryMessage.h>

#include "Platform.h"

#ifndef WIN32
#include <netinet/in.h>
#endif

OSG_BEGIN_NAMESPACE

/******************************************************************************
 * This message is an extension of the OpenSG BinaryMessage class.
 * It provides insert operations at the beginning of a message and
 * overrides the getUInt16 methods because of compatibility problems
 * with IRIX systems
 */
class INVRS_SYSTEMCORE_API ExtendedBinaryMessage : public BinaryMessage {
public:

	ExtendedBinaryMessage();
	virtual ~ExtendedBinaryMessage();
	ExtendedBinaryMessage(const ExtendedBinaryMessage &source);

	void putUInt64(const UInt64 value);
	UInt64 getUInt64();

	void putFirstUInt8(const UInt8 value);
	void putFirstUInt32(const UInt32 value);
	void putFirstUInt16(const UInt16 value);

	void appendMessage(ExtendedBinaryMessage* src);
	ExtendedBinaryMessage* detachMessage();

	bool finished();

protected:
	static const unsigned APPENDED_MESSAGE;
};

OSG_END_NAMESPACE

#endif
