/*---------------------------------------------------------------------------*\
 *           interactive networked Virtual Reality system (inVRs)            *
 *                                                                           *
 *    Copyright (C) 2005-2009 by the Johannes Kepler University, Linz        *
 *                                                                           *
 *                            www.inVRs.org                                  *
 *                                                                           *
 *              contact: canthes@inVRs.org, rlander@inVRs.org                *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                License                                    *
 *                                                                           *
 * This library is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU Library General Public License as published    *
 * by the Free Software Foundation, version 2.                               *
 *                                                                           *
 * This library is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Library General Public License for more details.                          *
 *                                                                           *
 * You should have received a copy of the GNU Library General Public         *
 * License along with this library; if not, write to the Free Software       *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                        Project: CAVESceneManager                          *
 *                                                                           *
 * The CAVE Scene Manager (CSM) is a wrapper for the OpenSG multi-display    *
 * functionality, which allows the easy configuration of a multi-display     *
 * setup via text files. It was developed by Adrian Haffegee of the Centre   *
 * for Advanced Computing and Emerging Technologies (ACET) at the            *
 * University of Reading, UK as part of his MSc Thesis.                      *
 *                                                                           *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                Changes                                    *
 *                                                                           *
 *                                                                           *
\*---------------------------------------------------------------------------*/


#ifndef _OSGCAVESCENEMANAGER_H_
#define _OSGCAVESCENEMANAGER_H_
#ifdef __sgi
#pragma once
#endif

#include <OpenSG/OSGBaseTypes.h>
#include <OpenSG/OSGWindow.h>

#include <OpenSG/OSGDrawAction.h>
#include <OpenSG/OSGRenderAction.h>
#include <OpenSG/OSGNode.h>
#include <OpenSG/OSGWindow.h>
#include <OpenSG/OSGImageForeground.h>
#include <OpenSG/OSGComponentTransform.h>
#include <OpenSG/OSGGeometry.h>
#include <OpenSG/OSGGeoPropPtrs.h>
#include <OpenSG/OSGSimpleMaterial.h>
#include <OpenSG/OSGPerspectiveCamera.h>
#include <OpenSG/OSGDirectionalLight.h>
#include <OpenSG/OSGStatisticsForeground.h>
#include <OpenSG/OSGShearedStereoCameraDecorator.h> // get rid of this if no longer needed
#include <OpenSG/OSGProjectionCameraDecorator.h>
#include <OpenSG/OSGStereoBufferViewport.h>
#include <OpenSG/OSGImageBackground.h>
#include <OpenSG/OSGCoredNodePtr.h>

//Shadow support
#include <OpenSG/OSGShadowViewport.h>
#include <OpenSG/OSGShadowMapViewport.h>

#include "OSGCAVEConfig.h"

#include <OpenSG/OSGTransform.h>

OSG_BEGIN_NAMESPACE

typedef CoredNodePtr<ComponentTransform> CTNodePtr;
//typedef CoredNodePtr<Transform> CTNodePtr;

/*! \brief A helper class to simplify managing simple applications.
    \ingroup GrpSystemLib
*/

class CAVESCENEMANAGER_DLLMAPPING CAVESceneManager
{
	/*==========================  PUBLIC  =================================*/
	public:
		/*---------------------------------------------------------------------*/
		/*! \name                      Enums                                   */
		/*! \{                                                                 */

		enum {  MouseLeft   =  0,
			MouseMiddle =  1,
			MouseRight  =  2,
			MouseUp     =  3,
			MouseDown   =  4
		};

		// ADDED BY HBRESS / LANDI
		void setFarClippingPlane(float farDist);
		void setNearClippingPlane(float nearDist);
		void setScale(float scale);

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                   Constructors                               */
		/*! \{                                                                 */

		CAVESceneManager(void);
		CAVESceneManager(CAVEConfig *cfg);
		CAVESceneManager *create( void );

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                   Destructor                                 */
		/*! \{                                                                 */

		virtual ~CAVESceneManager(void);

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                      Get                                     */
		/*! \{                                                                 */

		virtual NodePtr             getRoot            ( void );
		virtual WindowPtr           getWindow          ( void );
		virtual NodePtr             getHighlight       ( void );
		virtual Real32              getEyeSeparation   ( void );
		virtual DrawActionBase     *getAction          ( void );
		virtual bool                getHeadlightState  ( void );
		virtual Vec3f               getTranslation     ( void );
		virtual Real32              getYRotate         ( void );
		virtual NodePtr             getWand            ( void );
		virtual Quaternion             getWandRotate ( void );

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                      Set                                     */
		/*! \{                                                                 */

		virtual void  setRoot          ( NodePtr root );
		virtual void  setWindow        ( WindowPtr win );
		virtual void  setHighlight     ( NodePtr obj );
		virtual void  setEyeSeparation ( Real32 eyeSep );
		virtual void  setHeadlight     ( bool on );
		virtual void  turnHeadlightOn  ( void );
		virtual void  turnHeadlightOff ( void );
		virtual void  setTranslation   ( Vec3f Trans );
		virtual void  setRelativeTranslation   ( Vec3f Trans );
		virtual void  setRotation      ( Quaternion Rotation );
		virtual void  setBackground    ( Int16 position, ImagePtr image );
		virtual void  clearBackgrounds ( void );
		virtual void  setYRotate       ( Real32 rotation );
		virtual void  addYRotate       ( Real32 rotation );
		virtual int   setWand          ( NodePtr wandNode, int setPattern = 0);
		virtual void  setWandTranslation    ( Vec3f Trans );
		virtual void  setWandColour    ( Color3f colour );
		virtual void  setWandRotate    ( Quaternion rotation );
		virtual void  rotateWand (Quaternion rotation );
		virtual void  turnWandOn       ( void );
		virtual void  turnWandOff      ( void );

		virtual void  setUserTransform ( Vec3f trans, Quaternion rotate );
		virtual void  setSceneBoundaries   ( Vec3f Min, Vec3f Max );

		/* ADDED BY LANDI */
		virtual void  setCameraMatrix  ( Matrix m );

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name       Manipulating other objects in the scene                */
		/*! \{                                                                 */
		virtual bool  addToScene       ( NodePtr node, Int16 flags );
		virtual bool  removeFromScene  ( NodePtr node );
		virtual bool  setTranslationInScene (NodePtr node, Vec3f trans,
				Quaternion rotate );

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name       Manipulating objects attached to trolley                */
		/*! \{                                                                 */
		virtual bool  addToTrolley       ( NodePtr node, Int16 flags );
		virtual bool  removeFromTrolley  ( NodePtr node );
		virtual bool  setTranslationFromTrolley (NodePtr node, Vec3f trans,
				Quaternion rotate );


		virtual Vec3f validateNewTranslation(Vec3f desiredTrans);


		virtual void  showAll          ( void );

		virtual void  useOpenSGLogo    ( void );

		inline  bool  setClickCenter   ( bool mode );

		virtual void  setStatistics    ( bool on );
		virtual void  setForeground    ( Int16 position, ImageForegroundPtr fg, bool clear);

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name               Interaction handling                           */
		/*! \{                                                                 */

		virtual void resize(UInt16 width, UInt16 height);
		virtual void mouseMove(Int16 x, Int16 y);
		virtual void mouseButtonPress(UInt16 button, Int16 x, Int16 y);
		virtual void mouseButtonRelease(UInt16 button, Int16 x, Int16 y);
		virtual void key(UChar8 key, Int16 x, Int16 y);

		Line calcViewRay(Int16 x, Int16 y);

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                     Actions                                  */
		/*! \{                                                                 */

		virtual void redraw( void );
		virtual void idle( void );

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                    Comparison                                */
		/*! \{                                                                 */

		bool operator < (const CAVESceneManager &other) const;

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                    Shadow support                            */
		/*! \{                                                                 */
		virtual void enableShadows( void );
		virtual void addShadowLightSource(NodePtr light);
		virtual void removeShadowLightSource(NodePtr light);
		virtual void addExclusionFromShadow(NodePtr exclude);
		virtual void removeExclusionFromShadow(NodePtr include);
		virtual void setShadowOffBias(const Real32 &value);
		virtual void setShadowOffFactor(const Real32 &value);
		virtual void setShadowMapSize(const UInt32 &value);
		virtual void setShadowMode(const UInt32 &value);
		virtual void setShadowSmoothness(const Real32 &value);
		virtual void setShadowOn(const bool &value);
		virtual void setShadowAutoSearchForLights(const bool &value);
		virtual void setShadowGlobalShadowIntensity(const Real32 &value);
		virtual void setShadowAutoExcludeTransparentNodes(const bool &value);


		/*! \}                                                                 */
		/*=========================  PROTECTED  ===============================*/
	protected:

		/*---------------------------------------------------------------------*/
		/*! \name                     Updates                                  */
		/*! \{                                                                 */

		void  initialize      (void);
		virtual void  highlightChanged(void);
		virtual void  updateHighlight (void);

		/*! \}                                                                 */
		/*---------------------------------------------------------------------*/
		/*! \name                      Member                                  */
		/*! \{                                                                 */

		WindowPtr                    _win;
		NodePtr                      _root;

		ImageForegroundPtr           _foreground;
		StatisticsForegroundPtr      _statforeground;
		bool                         _statstate;

	    //Shadow support
	    bool                         _useShadows;

		NodePtr                      _highlight;
		NodePtr                      _highlightNode;
		GeoPositions3fPtr            _highlightPoints;

		NodePtr                      _internalRoot;
		NodePtr                      _litScene;
		NodePtr                      _wand;
		DirectionalLightPtr          _headlight;
		RenderAction *               _action;
		NodePtr                      _cameraTrolleyNode;
		ComponentTransformPtr        _camTrolleyTrans;
		PerspectiveCameraPtr         _cameras[MAXCAVEWALLS];
		CTNodePtr                    _wandCT;

		NodePtr                      _wandTransformNode;
		ComponentTransformPtr        _wandTransform;

		Vec3f                        _sceneBoundaryMin;
		Vec3f                        _sceneBoundaryMax;

		Int16                        _lastx;
		Int16                        _lasty;
		UInt16                       _mousebuttons;


		Real32                       _eyeseparation;
		TransformPtr                 _userTransform; // where the user is in relation to the screen
		Real32                       _yRotation;
		static  SimpleMaterialPtr    _highlightMaterial;
		CAVEConfig *                 _cfg;

		/* ADDED BY HBRESS,LANDI */
		Real32                       _scaleFactor;
		/*! \}                                                                 */
		/*==========================  PRIVATE  ================================*/
	private:

		/* prohibit default function (move to 'public' if needed) */

		CAVESceneManager(const CAVESceneManager &source);
		void operator =(const CAVESceneManager &source);

		ViewportPtr CreateMonoProjCamVeiwport(
				PerspectiveCameraPtr camera, Pnt4f size, ImageBackgroundPtr bg,
				NodePtr user, Pnt3f *vpproj);

		StereoBufferViewportPtr CreateProjStereoVeiwport(bool left,
				PerspectiveCameraPtr camera, Pnt4f size, ImageBackgroundPtr bg,
				NodePtr user, Pnt3f *vpproj);


		// Shadow support
		ShadowViewportPtr CreateMonoShadowViewport(
				PerspectiveCameraPtr camera, Pnt4f size, ImageBackgroundPtr bg,
				NodePtr user, Pnt3f *vpproj);
		ShadowViewportPtr CreateStereoShadowViewport(bool left,
				PerspectiveCameraPtr camera, Pnt4f size, ImageBackgroundPtr bg,
				NodePtr user, Pnt3f *vpproj);


	void CreateWand(NodePtr parentNode);

};

//---------------------------------------------------------------------------
//   Exported Types
//---------------------------------------------------------------------------

// class pointer

typedef CAVESceneManager *CAVESceneManagerP;

OSG_END_NAMESPACE


#define OSGCAVESCENEMANAGER_HEADER_CVSID "@(#)$Id: OSGCAVESceneManager.h,v 1.1.1.1 2004/10/07 01:01:10 adrian Exp $"

#endif /* _OSGCAVESCENEMANAGER_H_ */
