/*---------------------------------------------------------------------------*\
 *           interactive networked Virtual Reality system (inVRs)            *
 *                                                                           *
 *    Copyright (C) 2005-2009 by the Johannes Kepler University, Linz        *
 *                                                                           *
 *                            www.inVRs.org                                  *
 *                                                                           *
 *              contact: canthes@inVRs.org, rlander@inVRs.org                *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                License                                    *
 *                                                                           *
 * This library is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU Library General Public License as published    *
 * by the Free Software Foundation, version 2.                               *
 *                                                                           *
 * This library is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Library General Public License for more details.                          *
 *                                                                           *
 * You should have received a copy of the GNU Library General Public         *
 * License along with this library; if not, write to the Free Software       *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                        Project: CAVESceneManager                          *
 *                                                                           *
 * The CAVE Scene Manager (CSM) is a wrapper for the OpenSG multi-display    *
 * functionality, which allows the easy configuration of a multi-display     *
 * setup via text files. It was developed by Adrian Haffegee of the Centre   *
 * for Advanced Computing and Emerging Technologies (ACET) at the            *
 * University of Reading, UK as part of his MSc Thesis.                      *
 *                                                                           *
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
 *                                Changes                                    *
 *                                                                           *
 *                                                                           *
\*---------------------------------------------------------------------------*/


#ifndef _CVR_CAVEWALL_H_
#define _CVR_CAVEWALL_H_

#include <OpenSG/OSGBaseTypes.h>
#include <OpenSG/OSGGeometry.h>

OSG_BEGIN_NAMESPACE


/*! \brief CAVE wall display class
 */
class CAVEWall {
	public:

		enum wallPos{ wallFront   =  0, /**< front wall */
			wallLeft,					/**< left wall */
			wallRight,					/**< right wall */
			wallFloor,					/**< floor wall */
			wallBack,					/**< back wall */
			wallCeiling,				/**< ceiling wall */
			wallUserDefined				/**< user defined wall */
		};


		CAVEWall(char *name);
		virtual ~CAVEWall(void);

		void printConfig();
		bool checkProjectionData(Pnt3f origin, float width, float height);

		void setActive(bool activate);
		bool setPipe(char *pipe);
		void setWindow(bool window);
		bool setGeometry(char *geometry);
		void setPipeIndex(int index);
		void setProjectionData(Pnt3f lowerLeft, Pnt3f upperLeft, Pnt3f lowerRight, float convFact);

		char* getName();
		bool isActive();
		char* getPipe();
		bool getWindow(void);
		char* getGeometry();
		int getGeomCoordMinX();
		int getGeomCoordMaxX();
		int getGeomCoordMinY();
		int getGeomCoordMaxY();
		Pnt3f *getProjData();
		int getPipeIndex();

	private:

		char *m_name;
		bool m_active;
		char* m_pipe;
		bool m_window;
		char* m_geometry;
		wallPos m_wallPos;
		Pnt3f m_projData[4];
		int m_pipeIndex;
		int m_geomCoordMinX;
		int m_geomCoordMaxX;
		int m_geomCoordMinY;
		int m_geomCoordMaxY;
};

OSG_END_NAMESPACE

#endif // _CVR_CAVEWALL_H_
